@@ -8,10 +8,6 @@
 ##  starts at 1, each new drive, numbers shared across both teams
 ## fixed_drive_result =
 ##  result of  given drive
-
-#' @import dplyr
-#' @importFrom rlang .data
-#' @importFrom stats na.omit
 add_drive_results <- function(d) {
   drive_df <- d %>%
     dplyr::group_by(.data$game_id, .data$game_half) %>%
@@ -35,6 +31,15 @@ add_drive_results <- function(d) {
           & !is.na(dplyr::lag(.data$posteam)),
         0,
         .data$new_drive),
+      # if same team has the ball as prior play, but prior play was a punt with lost fumble, it's a new drive
+      new_drive = dplyr::if_else(
+        # this line is to prevent it from overwriting already-defined new drives with NA
+        # when there's a timeout on prior line
+        .data$new_drive != 1 &
+          # same team has ball after lost fumble on punt
+          .data$posteam == dplyr::lag(.data$posteam) & dplyr::lag(.data$fumble_lost == 1) & dplyr::lag(.data$play_type) == "punt",
+        1, .data$new_drive
+      ),
       # first observation of a half is also a new drive
       new_drive = dplyr::if_else(.data$row == 1, 1, .data$new_drive),
       # if there's a missing, make it not a new drive (0)
@@ -49,8 +54,8 @@ add_drive_results <- function(d) {
         .data$field_goal_result == "made" ~ "Field goal",
         .data$field_goal_result %in% c("blocked", "missed") ~ "Missed field goal",
         .data$safety == 1 ~ "Safety",
-        .data$interception == 1 | .data$fumble_lost == 1 ~ "Turnover",
         .data$play_type == "punt" | .data$punt_attempt == 1 ~ "Punt",
+        .data$interception == 1 | .data$fumble_lost == 1 ~ "Turnover",
         .data$down == 4 & .data$yards_gained < .data$ydstogo & .data$play_type != "no_play" ~ "Turnover on downs",
         .data$desc %in% c("END GAME", "END QUARTER 2", "END QUARTER 4") ~ "End of half"
       )
