@@ -16,8 +16,6 @@
 #' \item{xyac_mean_yardage}{Average expected yards after the catch based on where the ball was caught.}
 #' \item{xyac_median_yardage}{Median expected yards after the catch based on where the ball was caught.}
 #' }
-#' @importFrom rlang .data
-#' @importFrom xgboost getinfo
 #' @export
 add_xyac <- function(pbp, ...) {
 
@@ -27,7 +25,13 @@ add_xyac <- function(pbp, ...) {
     # testing only
     # pbp <- g
 
-    usethis::ui_todo("Computing xyac...")
+    # Need this ifelse because the else won't render correct on the pkgdown website
+    # but we don't want crayon to be a hard dependency
+    if (is_installed("crayon")){
+      rlang::inform(paste0(crayon::red(cli::symbol$bullet), " Computing xyac... (", crayon::yellow(cli::symbol$info), " This is a heavy task, please be patient)"))
+    } else {
+      rlang::inform(paste0("\033[31m*\033[39m", " Computing xyac... (", "\033[33mi\033[39m", " This is a heavy task, please be patient)"))
+    }
 
     pbp <- pbp %>% dplyr::select(-tidyselect::any_of(drop.cols.xyac))
 
@@ -37,29 +41,18 @@ add_xyac <- function(pbp, ...) {
 
     # prepare_xyac_data helper function shown below
     passes <- prepare_xyac_data(pbp) %>%
-      filter(.data$valid_pass == 1, .data$distance_to_goal != 0)
+      dplyr::filter(.data$valid_pass == 1, .data$distance_to_goal != 0)
 
     if (!nrow(passes) == 0) {
-      # initialize xyac_model to avoid R CMD check note
-      xyac_model <- NULL
-      suppressWarnings(
-        # load the model from github because it is too big for the package
-        try(
-          load(url("https://github.com/guga31bb/nflfastR-data/blob/master/models/xyac_model.Rdata?raw=true")),
-          silent = TRUE
-        )
-      )
-
-      if (!is.null(xyac_model)) {
         xyac_vars <-
           stats::predict(
-            xyac_model,
+            fastrmodels::xyac_model,
             as.matrix(passes %>% xyac_model_select())
           ) %>%
           tibble::as_tibble() %>%
           dplyr::rename(prob = "value") %>%
           dplyr::bind_cols(
-            purrr::map_dfr(seq_along(passes$index), function(x) {
+            furrr::future_map_dfr(seq_along(passes$index), function(x) {
               tibble::tibble(
                 "yac" = -5:70,
                 "index" = passes$index[[x]],
@@ -145,8 +138,7 @@ add_xyac <- function(pbp, ...) {
             xyac_mean_yardage = (dplyr::first(.data$original_spot) - dplyr::first(.data$air_yards)) - sum(.data$wt_yardln),
             xyac_median_yardage = max(.data$med),
             xyac_success = sum((.data$ep > .data$original_ep) * .data$prob),
-            xyac_fd = sum((.data$gain >= .data$original_ydstogo) * .data$prob),
-            .groups = "drop_last"
+            xyac_fd = sum((.data$gain >= .data$original_ydstogo) * .data$prob)
           ) %>%
           dplyr::ungroup()
 
@@ -156,10 +148,6 @@ add_xyac <- function(pbp, ...) {
 
         message_completed("added xyac variables", ...)
 
-      } else {# means xyac_model isn't available
-        usethis::ui_oops("This function needs to download the model data from GitHub. Please check your Internet connection and try again!")
-        pbp <- pbp %>% dplyr::select(-.data$index)
-      }
     } else {# means no valid pass plays in the pbp
       pbp <- pbp %>%
         dplyr::mutate(
@@ -172,9 +160,6 @@ add_xyac <- function(pbp, ...) {
         dplyr::select(-.data$index)
       usethis::ui_info("No non-NA values for xyac calculation detected. xyac variables set to NA")
     }
-
-    # on old versions of dplyr, a .groups column is created, which we don't want
-    pbp <- pbp %>% dplyr::select(-tidyselect::any_of(".groups"))
   }
 
   return(pbp)
@@ -218,7 +203,7 @@ xyac_model_select <- function(pbp) {
 # These columns are being generated by add_xyac and the function tries to drop
 # them in case it is being used on a pbp dataset where the columns already exist
 drop.cols.xyac <- c(
-  "xyac_epa", "xyac_mean_yardage", "xyac_median_yardage", "xyac_success", "xyac_fd", ".groups"
+  "xyac_epa", "xyac_mean_yardage", "xyac_median_yardage", "xyac_success", "xyac_fd"
 )
 
 
