@@ -5,18 +5,18 @@
 
 #' Clean Play by Play Data
 #'
-#' @param pbp is a Data frame of play-by-play data scraped using \code{\link{fast_scraper}}.
+#' @param pbp is a Data frame of play-by-play data scraped using [fast_scraper()].
 #' @param ... Additional arguments passed to a message function (for internal use).
 #' @details Build columns that capture what happens on all plays, including
 #' penalties, using string extraction from play description.
-#' Loosely based on Ben's nflfastR guide (\url{https://www.nflfastr.com/articles/beginners_guide.html})
+#' Loosely based on Ben's nflfastR guide (<https://www.nflfastr.com/articles/beginners_guide.html>)
 #' but updated to work with the RS data, which has a different player format in
 #' the play description; e.g. 24-M.Lynch instead of M.Lynch.
 #' The function also standardizes team abbreviations so that, for example,
 #' the Chargers are always represented by 'LAC' regardless of which year it was.
-#' The function also standardizes player IDs for players appearing in both the
-#' older era (1999-2010) and the new era (2011+).
-#' @return The input Data Frame of the paramter 'pbp' with the following columns
+#' @seealso For information on parallel processing and progress updates please
+#' see [nflfastR].
+#' @return The input Data Frame of the parameter 'pbp' with the following columns
 #' added:
 #' \describe{
 #' \item{success}{Binary indicator wheter epa > 0 in the given play. }
@@ -32,30 +32,26 @@
 #' \item{first_down}{Binary indicator if the play ended in a first down.}
 #' \item{aborted_play}{Binary indicator if the play description indicates "Aborted".}
 #' \item{play}{Binary indicator: 1 if the play was a 'normal' play (including penalties), 0 otherwise.}
-#' \item{passer_id}{ID of the player in the 'passer' column (NOTE: ids vary pre and post 2011 but are consistent for each player. Please see details for further information)}
-#' \item{rusher_id}{ID of the player in the 'rusher' column (NOTE: ids vary pre and post 2011 but are consistent for each player. Please see details for further information)}
-#' \item{receiver_id}{ID of the player in the 'receiver' column (NOTE: ids vary pre and post 2011 but are consistent for each player. Please see details for further information)}
+#' \item{passer_id}{ID of the player in the 'passer' column.}
+#' \item{rusher_id}{ID of the player in the 'rusher' column.}
+#' \item{receiver_id}{ID of the player in the 'receiver' column.}
 #' \item{name}{Name of the 'passer' if it is not 'NA', or name of the 'rusher' otherwise.}
+#' \item{fantasy}{Name of the rusher on rush plays or receiver on pass plays.}
+#' \item{fantasy_id}{ID of the rusher on rush plays or receiver on pass plays.}
+#' \item{fantasy_player_name}{Name of the rusher on rush plays or receiver on pass plays (from official stats).}
+#' \item{fantasy_player_id}{ID of the rusher on rush plays or receiver on pass plays (from official stats).}
 #' \item{jersey_number}{Jersey number of the player listed in the 'name' column.}
-#' \item{id}{ID of the player in the 'name' column (NOTE: ids vary pre and post 2011 but are consistent for each player. Please see details for further information)}
+#' \item{id}{ID of the player in the 'name' column.}
 #' \item{qb_epa}{Gives QB credit for EPA for up to the point where a receiver lost a fumble after a completed catch and makes EPA work more like passing yards on plays with fumbles.}
+#' \item{out_of_bounds}{1 if play description contains ran ob, pushed ob, or sacked ob; 0 otherwise.}
 #' }
 #' @export
-#' @import dplyr
-#' @importFrom stringr str_detect str_extract str_replace_all
-#' @importFrom glue glue
-#' @importFrom rlang .data
-#' @importFrom tidyselect any_of
 clean_pbp <- function(pbp, ...) {
   if (nrow(pbp) == 0) {
     usethis::ui_info("Nothing to clean. Return passed data frame.")
     r <- pbp
   } else{
-    rlang::inform(paste0(crayon::red(cli::symbol$bullet), " Cleaning up play-by-play... (", crayon::yellow(cli::symbol$info), " If you run this with a lot of seasons this could take a few minutes.)"))
-
-    # Load id map to standardize player ids for players that were active before 2011
-    # and in or after 2011 meaning they appear with old gsis_ids and new ids
-    legacy_id_map <- readRDS(url("https://github.com/guga31bb/nflfastR-data/blob/master/roster-data/legacy_id_map.rds?raw=true"))
+    usethis::ui_todo("Cleaning up play-by-play...")
 
     # drop existing values of clean_pbp
     pbp <- pbp %>% dplyr::select(-tidyselect::any_of(drop.cols))
@@ -96,15 +92,19 @@ clean_pbp <- function(pbp, ...) {
         ),
         #if no pass is thrown, there shouldn't be a receiver
         receiver = dplyr::if_else(
-          stringr::str_detect(.data$desc, ' pass'), .data$receiver, NA_character_
+          stringr::str_detect(.data$desc, ' pass '), .data$receiver, NA_character_
         ),
         # if there's a pass, sack, or scramble, it's a pass play...
-        pass = dplyr::if_else(stringr::str_detect(.data$desc, "( pass)|(sacked)|(scramble)"), 1, 0),
+        pass = dplyr::if_else(stringr::str_detect(.data$desc, "( pass )|(sacked)|(scramble)"), 1, 0),
         # ...unless it says "backwards pass" and there's a rusher
         pass = dplyr::if_else(
           stringr::str_detect(.data$desc, "(backward pass)|(Backward pass)") & !is.na(.data$rusher),
           0, .data$pass
           ),
+        # and make sure there's no pass on a kickoff (sometimes there's forward pass on kickoff but that's not a pass play)
+        pass = dplyr::if_else(
+          .data$kickoff_attempt == 1, 0, .data$pass
+        ),
         #if there's a rusher and it wasn't a QB kneel or pass play, it's a run play
         rush = dplyr::if_else(!is.na(.data$rusher) & .data$qb_kneel == 0 & .data$pass == 0, 1, 0),
         #fix some common QBs with inconsistent names
@@ -205,11 +205,36 @@ clean_pbp <- function(pbp, ...) {
         jersey_number = dplyr::if_else(!is.na(.data$passer_jersey_number), .data$passer_jersey_number, .data$rusher_jersey_number),
         id = dplyr::if_else(!is.na(.data$passer_id), .data$passer_id, .data$rusher_id)
       ) %>%
-      dplyr::mutate_at(
-        dplyr::vars(.data$passer_id, .data$rusher_id, .data$receiver_id, .data$id, ends_with("player_id")),
-        update_ids, legacy_id_map) %>%
       dplyr::arrange(.data$index) %>%
-      dplyr::select(-"index")
+      dplyr::select(-"index") %>%
+      # add action player
+      dplyr::mutate(
+        fantasy_player_name = case_when(
+          !is.na(.data$rusher_player_name) ~ .data$rusher_player_name,
+          is.na(.data$rusher_player_name) & !is.na(.data$receiver_player_name) ~ .data$receiver_player_name,
+          TRUE ~ NA_character_
+        ),
+        fantasy_player_id = case_when(
+          !is.na(.data$rusher_player_id) ~ .data$rusher_player_id,
+          is.na(.data$rusher_player_id) & !is.na(.data$receiver_player_id) ~ .data$receiver_player_id,
+          TRUE ~ NA_character_
+        ),
+        fantasy = case_when(
+          !is.na(.data$rusher) ~ .data$rusher,
+          is.na(.data$rusher) & !is.na(.data$receiver) ~ .data$receiver,
+          .data$qb_scramble == 1 ~ .data$passer,
+          TRUE ~ NA_character_
+        ),
+        fantasy_id = case_when(
+          !is.na(.data$rusher_id) ~ .data$rusher_id,
+          is.na(.data$rusher_id) & !is.na(.data$receiver_id) ~ .data$receiver_id,
+          .data$qb_scramble == 1 ~ .data$passer_id,
+          TRUE ~ NA_character_
+        ),
+        out_of_bounds = dplyr::if_else(
+          stringr::str_detect(.data$desc, "(ran ob)|(pushed ob)|(sacked ob)"), 1, 0
+        )
+      )
   }
 
   message_completed("Cleaning completed", ...)
@@ -264,31 +289,13 @@ team_name_fn <- function(var) {
   )
 }
 
-#' @importFrom tibble tibble
-#' @importFrom rlang .data
-#' @importFrom dplyr left_join mutate case_when
-update_ids <- function(var, id_map) {
-  join <- tibble::tibble(id = var) %>%
-    dplyr::left_join(id_map, by = c("id" = "gsis_id")) %>%
-    dplyr::mutate(
-      out_id = dplyr::case_when(
-        is.na(.data$new_id) ~ .data$id,
-        TRUE ~ .data$new_id
-      )
-    )
-  return(join$out_id)
-}
-
 #' Compute QB epa
 #'
-#'@inheritParams clean_pbp
+#' @inheritParams clean_pbp
 #' @details Add the variable 'qb_epa', which gives QB credit for EPA for up to the point where
 #' a receiver lost a fumble after a completed catch and makes EPA work more
 #' like passing yards on plays with fumbles
 #' @export
-#' @import dplyr
-#' @importFrom rlang .data
-#' @importFrom tidyselect any_of
 add_qb_epa <- function(pbp, ...) {
 
   if (nrow(pbp) == 0) {
