@@ -12,32 +12,10 @@
 # @param play_Id (integer) Specifies the play_Id for which the stats should be combined
 # @param stats A dataframe including multiple rows for each play_Id holding
 # gsis stat ids and stats
-#' @importFrom dplyr filter if_else
-#' @importFrom tibble as_tibble_row
-#' @importFrom purrr prepend set_names modify_at modify_if
-#' @importFrom rlang .data
 sum_play_stats <- function(play_Id, stats) {
   play_stats <- stats %>% filter(.data$playId == play_Id)
 
-  row <-
-    as_tibble_row(
-      matrix(NA,ncol = length(pbp_stat_columns) - 1),
-      .name_repair = "minimal"
-    ) %>%
-    prepend(as.integer(play_Id)) %>%
-    set_names(pbp_stat_columns) %>%
-    modify_at(indicator_stats, function(x) {
-      x <- 0
-    }) %>%
-    modify_if(is.na, function(x) {
-      x <- NA_character_
-    }) %>%
-    modify_at(c(
-      "air_yards", "yards_after_catch", "penalty_yards", "kick_distance",
-      "fumble_recovery_1_yards", "fumble_recovery_2_yards"
-    ), as.integer) %>%
-    as_tibble_row()
-
+  row <- bind_cols(play_id = as.integer(play_Id), tidy_play_stats_row)
 
   for (index in seq_along(play_stats$playId)) {
     stat_id <- play_stats$statId[index]
@@ -64,6 +42,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$rusher_player_id <- play_stats$player.esbId[index]
       row$rusher_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$rushing_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 11) {
       row$rush_attempt <- 1
@@ -73,6 +52,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$rusher_player_id <- play_stats$player.esbId[index]
       row$rusher_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$rushing_yards <- play_stats$yards[index]
       row$td_team <- play_stats$teamAbbr[index]
       row$penalty_fix <- 1
     } else if (stat_id == 12) {
@@ -81,6 +61,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$lateral_rusher_player_id <- play_stats$player.esbId[index]
       row$lateral_rusher_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$lateral_rushing_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 13) {
       row$rush_attempt <- 1
@@ -91,6 +72,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$lateral_rusher_player_name <- play_stats$player.displayName[index]
       row$td_team <- play_stats$teamAbbr[index]
       row$yards_gained <- play_stats$yards[index]
+      row$lateral_rushing_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 14) {
       row$incomplete_pass <- 1
@@ -104,6 +86,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$passer_player_id <- play_stats$player.esbId[index]
       row$passer_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$passing_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 16) {
       row$pass_attempt <- 1
@@ -113,6 +96,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$passer_player_id <- play_stats$player.esbId[index]
       row$passer_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$passing_yards <- play_stats$yards[index]
       row$td_team <- play_stats$teamAbbr[index]
       row$penalty_fix <- 1
     } else if (stat_id == 19) {
@@ -134,6 +118,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$receiver_player_id <- play_stats$player.esbId[index]
       row$receiver_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$receiving_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 22) {
       row$pass_attempt <- 1
@@ -144,6 +129,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$receiver_player_name <- play_stats$player.displayName[index]
       row$td_team <- play_stats$teamAbbr[index]
       row$yards_gained <- play_stats$yards[index]
+      row$receiving_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 23) {
       row$pass_attempt <- 1
@@ -152,6 +138,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$lateral_receiver_player_id <- play_stats$player.esbId[index]
       row$lateral_receiver_player_name <- play_stats$player.displayName[index]
       row$yards_gained <- play_stats$yards[index]
+      row$lateral_receiving_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 24) {
       row$pass_attempt <- 1
@@ -163,6 +150,7 @@ sum_play_stats <- function(play_Id, stats) {
       row$lateral_receiver_player_name <- play_stats$player.displayName[index]
       row$td_team <- play_stats$teamAbbr[index]
       row$yards_gained <- play_stats$yards[index]
+      row$lateral_receiving_yards <- play_stats$yards[index]
       row$penalty_fix <- 1
     } else if (stat_id == 25) {
       row$pass_attempt <- 1
@@ -1249,270 +1237,3 @@ sum_play_stats <- function(play_Id, stats) {
   }
   return(row)
 }
-
-
-
-# stats character vectors -------------------------------------------------
-
-pbp_stat_columns <-
-  c(
-    "play_id",
-    "punt_blocked",
-    "first_down_rush",
-    "first_down_pass",
-    "first_down_penalty",
-    "third_down_converted",
-    "third_down_failed",
-    "fourth_down_converted",
-    "fourth_down_failed",
-    "incomplete_pass",
-    "interception",
-    "punt_inside_twenty",
-    "punt_in_endzone",
-    "punt_out_of_bounds",
-    "punt_downed",
-    "punt_fair_catch",
-    "kickoff_inside_twenty",
-    "kickoff_in_endzone",
-    "kickoff_out_of_bounds",
-    "kickoff_fair_catch",
-    "fumble_forced",
-    "fumble_not_forced",
-    "fumble_out_of_bounds",
-    "timeout",
-    "field_goal_missed",
-    "field_goal_made",
-    "field_goal_blocked",
-    "extra_point_good",
-    "extra_point_failed",
-    "extra_point_blocked",
-    "two_point_rush_good",
-    "two_point_rush_failed",
-    "two_point_pass_good",
-    "two_point_pass_failed",
-    "solo_tackle",
-    "safety",
-    "penalty",
-    "tackled_for_loss",
-    "extra_point_safety",
-    "two_point_rush_safety",
-    "two_point_pass_safety",
-    "kickoff_downed",
-    "two_point_pass_reception_good",
-    "two_point_pass_reception_failed",
-    "fumble_lost",
-    "own_kickoff_recovery",
-    "own_kickoff_recovery_td",
-    "qb_hit",
-    "extra_point_aborted",
-    "two_point_return",
-    "rush_attempt",
-    "pass_attempt",
-    "sack",
-    "touchdown",
-    "pass_touchdown",
-    "rush_touchdown",
-    "return_touchdown",
-    "extra_point_attempt",
-    "two_point_attempt",
-    "field_goal_attempt",
-    "kickoff_attempt",
-    "punt_attempt",
-    "fumble",
-    "complete_pass",
-    "assist_tackle",
-    "lateral_reception",
-    "lateral_rush",
-    "lateral_return",
-    "lateral_recovery",
-    "passer_player_id",
-    "passer_player_name",
-    "receiver_player_id",
-    "receiver_player_name",
-    "rusher_player_id",
-    "rusher_player_name",
-    "lateral_receiver_player_id",
-    "lateral_receiver_player_name",
-    "lateral_rusher_player_id",
-    "lateral_rusher_player_name",
-    "lateral_sack_player_id",
-    "lateral_sack_player_name",
-    "interception_player_id",
-    "interception_player_name",
-    "lateral_interception_player_id",
-    "lateral_interception_player_name",
-    "punt_returner_player_id",
-    "punt_returner_player_name",
-    "lateral_punt_returner_player_id",
-    "lateral_punt_returner_player_name",
-    "kickoff_returner_player_name",
-    "kickoff_returner_player_id",
-    "lateral_kickoff_returner_player_id",
-    "lateral_kickoff_returner_player_name",
-    "punter_player_id",
-    "punter_player_name",
-    "kicker_player_name",
-    "kicker_player_id",
-    "own_kickoff_recovery_player_id",
-    "own_kickoff_recovery_player_name",
-    "blocked_player_id",
-    "blocked_player_name",
-    "tackle_for_loss_1_player_id",
-    "tackle_for_loss_1_player_name",
-    "tackle_for_loss_2_player_id",
-    "tackle_for_loss_2_player_name",
-    "qb_hit_1_player_id",
-    "qb_hit_1_player_name",
-    "qb_hit_2_player_id",
-    "qb_hit_2_player_name",
-    "forced_fumble_player_1_team",
-    "forced_fumble_player_1_player_id",
-    "forced_fumble_player_1_player_name",
-    "forced_fumble_player_2_team",
-    "forced_fumble_player_2_player_id",
-    "forced_fumble_player_2_player_name",
-    "solo_tackle_1_team",
-    "solo_tackle_2_team",
-    "solo_tackle_1_player_id",
-    "solo_tackle_2_player_id",
-    "solo_tackle_1_player_name",
-    "solo_tackle_2_player_name",
-    "assist_tackle_1_player_id",
-    "assist_tackle_1_player_name",
-    "assist_tackle_1_team",
-    "assist_tackle_2_player_id",
-    "assist_tackle_2_player_name",
-    "assist_tackle_2_team",
-    "assist_tackle_3_player_id",
-    "assist_tackle_3_player_name",
-    "assist_tackle_3_team",
-    "assist_tackle_4_player_id",
-    "assist_tackle_4_player_name",
-    "assist_tackle_4_team",
-    # new for stat ID 80 -> tackle_with_assist
-    "tackle_with_assist_1_player_id",
-    "tackle_with_assist_1_player_name",
-    "tackle_with_assist_1_team",
-    "tackle_with_assist_2_player_id",
-    "tackle_with_assist_2_player_name",
-    "tackle_with_assist_2_team",
-
-    "pass_defense_1_player_id",
-    "pass_defense_1_player_name",
-    "pass_defense_2_player_id",
-    "pass_defense_2_player_name",
-    "fumbled_1_team",
-    "fumbled_1_player_id",
-    "fumbled_1_player_name",
-    "fumbled_2_player_id",
-    "fumbled_2_player_name",
-    "fumbled_2_team",
-    "fumble_recovery_1_team",
-    "fumble_recovery_1_yards",
-    "fumble_recovery_1_player_id",
-    "fumble_recovery_1_player_name",
-    "fumble_recovery_2_team",
-    "fumble_recovery_2_yards",
-    "fumble_recovery_2_player_id",
-    "fumble_recovery_2_player_name",
-    "td_team",
-    "return_team",
-    "timeout_team",
-    "yards_gained",
-    "return_yards",
-    "air_yards",
-    "yards_after_catch",
-    "penalty_team",
-    "penalty_player_id",
-    "penalty_player_name",
-    "penalty_yards",
-    "kick_distance",
-    "defensive_two_point_attempt",
-    "defensive_two_point_conv",
-    "defensive_extra_point_attempt",
-    "defensive_extra_point_conv",
-    "penalty_fix",
-    "return_penalty_fix"
-  )
-
-indicator_stats <- c(
-  "punt_blocked",
-  "first_down_rush",
-  "first_down_pass",
-  "first_down_penalty",
-  "third_down_converted",
-  "third_down_failed",
-  "fourth_down_converted",
-  "fourth_down_failed",
-  "incomplete_pass",
-  "interception",
-  "punt_inside_twenty",
-  "punt_in_endzone",
-  "punt_out_of_bounds",
-  "punt_downed",
-  "punt_fair_catch",
-  "kickoff_inside_twenty",
-  "kickoff_in_endzone",
-  "kickoff_out_of_bounds",
-  "kickoff_fair_catch",
-  "fumble_forced",
-  "fumble_not_forced",
-  "fumble_out_of_bounds",
-  "timeout",
-  "field_goal_missed",
-  "field_goal_made",
-  "field_goal_blocked",
-  "extra_point_good",
-  "extra_point_failed",
-  "extra_point_blocked",
-  "two_point_rush_good",
-  "two_point_rush_failed",
-  "two_point_pass_good",
-  "two_point_pass_failed",
-  "solo_tackle",
-  "safety",
-  "penalty",
-  "tackled_for_loss",
-  "extra_point_safety",
-  "two_point_rush_safety",
-  "two_point_pass_safety",
-  "kickoff_downed",
-  "two_point_pass_reception_good",
-  "two_point_pass_reception_failed",
-  "fumble_lost",
-  "own_kickoff_recovery",
-  "own_kickoff_recovery_td",
-  "qb_hit",
-  "extra_point_aborted",
-  "two_point_return",
-  "defensive_two_point_attempt",
-  "defensive_two_point_conv",
-  "defensive_extra_point_attempt",
-  "defensive_extra_point_conv",
-  "rush_attempt",
-  "pass_attempt",
-  "sack",
-  "touchdown",
-  "pass_touchdown",
-  "rush_touchdown",
-  "return_touchdown",
-  "extra_point_attempt",
-  "two_point_attempt",
-  "field_goal_attempt",
-  "kickoff_attempt",
-  "punt_attempt",
-  "fumble",
-  "complete_pass",
-  "assist_tackle",
-  # new for stat ID 80 -> tackle_with_assist
-  "tackle_with_assist",
-
-  "lateral_reception",
-  "lateral_rush",
-  "lateral_return",
-  "lateral_recovery",
-  "penalty_fix",
-  "yards_gained",
-  "return_yards",
-  "return_penalty_fix"
-)
